/*
 * Copyright (C) 2003 Sebastian Krahmer.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Sebastian Krahmer.
 * 4. The name Sebastian Krahmer may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#include <iostream>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <errno.h>
#include <cstdio>
#include <arpa/inet.h>
#include <map>
#include "packet.h"
#include "misc.h"


using namespace loaded;
using namespace std;

int do_balance(int af)
{
	Packet *p = new Packet;
	sockaddr_in b4;
	sockaddr_in6 b6;

	p->init();

	for (;;) {
		if (p->recv() < 0) {
			cerr<<p->why()<<endl;
			return -1;
		}
		if (p->balance(b4, b6, af) < 0) {
			cerr<<p->why()<<endl;
			return -1;
		}

		if (af == AF_INET) {
			p->send((sockaddr*)&b4, af);
		} else if (af == AF_INET6) {
			p->send((sockaddr*)&b6, af);
		}
	}
	return 0;
}


namespace loaded {

extern map<sockaddr_in, size_t, sockaddrLess> packetCount4;

}


void usage(const char *path)
{
	cout<<"Usage: "<<path<<" [-4 | -6] [-c configfile]\n";
	exit(1);
}


int main(int argc, char **argv)
{
	string config = "loaded.config";
	int i = 0, af = AF_INET;

	while ((i = getopt(argc, argv, "46c:")) != -1) {
		switch (i) {
		case '4':
			af = AF_INET;
			break;
		case '6':
			af = AF_INET6;
			break;
		case 'c':
			config = optarg;
			break;
		default:
			usage(argv[0]);
		}
	}

	parse_config(config);
	do_balance(af);
	return 0;
}

