#include <cstdio>
#include <cerrno>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <map>
#include "packet.h"
#include "strategy.h"

namespace loaded {

using namespace std;


unsigned short
in_cksum (unsigned short *ptr, int nbytes)
{

	register long sum;		/* assumes long == 32 bits */
	u_short oddbyte;
	register u_short answer;	/* assumes u_short == 16 bits */


	/*
	* Our algorithm is simple, using a 32-bit accumulator (sum),
	* we add sequential 16-bit words to it, and at the end, fold back
	* all the carry bits from the top 16 bits into the lower 16 bits.
	*/

	sum = 0;
	while (nbytes > 1)
	{
	sum += *ptr++;
	nbytes -= 2;
	}

	/* mop up an odd byte, if necessary */
	if (nbytes == 1)
	{
	oddbyte = 0;		/* make sure top half is zero */
	*((unsigned char *) & oddbyte) = *(unsigned char *) ptr;	/* one byte only */
	sum += oddbyte;
	}

	/*
	* Add back carry outs from top 16 bits to low 16 bits.
	*/

	sum = (sum >> 16) + (sum & 0xffff);	/* add high-16 to low-16 */
	sum += (sum >> 16);		/* add carry */
	answer = ~sum;		/* ones-complement, then truncate to 16 bits */
	return (answer);
}



extern sockaddr_in VIP4;
extern sockaddr_in6 VIP6;
string strategy;

int parse_config(const string &path)
{
	FILE *f = fopen(path.c_str(), "r");
	if (!f) {
		fprintf(stderr, "parse_config: Can't open file %s: %s\n",
		        path.c_str(), strerror(errno));
		return -1;
	}

	char buf[1024], *ptr = NULL;
	while (fgets(buf, sizeof(buf), f)) {
		ptr = buf;
		while (*ptr == ' ' || *ptr == '\t')
			++ptr;
		if (*ptr == '#')
			continue;
		if (strncmp(ptr, "server4", 7) == 0) {
			ptr += 8;
			strtok(ptr, " \t\n#");
			sockaddr_in sin;
			memset(&sin, 0, sizeof(sin));
			if (inet_pton(AF_INET, ptr, &sin.sin_addr) < 0) {
				fprintf(stderr, "parse_config::inet_pton: %s",
				        strerror(errno));
				return -1;
			}
			packetCount4[sin] = 0;
		} if (strncmp(ptr, "server6", 7) == 0) {
			ptr += 8;
			strtok(ptr, " \t\n#");
			sockaddr_in6 sin;
			memset(&sin, 0, sizeof(sin));
			if (inet_pton(AF_INET6, ptr, &sin.sin6_addr) < 0) {
				fprintf(stderr, "parse_config::inet_pton: %s",
				        strerror(errno));
				return -1;
			}
			packetCount6[sin] = 0;
		
		} else if (strncmp(ptr, "VIP4", 4) == 0) {
			ptr += 5;
			strtok(ptr, " \t\n#");
			if (inet_pton(AF_INET, ptr, &VIP4.sin_addr) < 0) {
				fprintf(stderr, "parse_config::inet_pton: %s",
				        strerror(errno));
				return -1;
			}
		} else if (strncmp(ptr, "VIP6", 4) == 0) {
			ptr += 5;
			strtok(ptr, " \t\n#");
			if (inet_pton(AF_INET6, ptr, &VIP6.sin6_addr) < 0) {
				fprintf(stderr, "parse_config::inet_pton: %s", strerror(errno));
				return -1;
			}
	
		} else if (strncmp(ptr, "strategy", 8) == 0) {
			ptr += 9;
			strtok(ptr, " \t\n#");
			strategy = ptr;
		}
	}
	fclose(f);
	return 0;

}


} // namespace loaded


