/*
 * Copyright (C) 2003 Sebastian Krahmer.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Sebastian Krahmer.
 * 4. The name Sebastian Krahmer may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#ifndef __packet_h__
#define __packet_h__

#include <sys/socket.h>
#include <netinet/in.h>
#include <string>
#include <map>
extern "C" {
#include <linux/netfilter.h>
#include <libipq.h>
}

#include "misc.h"

namespace loaded {

#define MTU 65000 

struct sockaddrLess
{
	bool operator()(sockaddr_in s1, sockaddr_in s2)
	{
		unsigned int i1, i2;
		i1 = *(unsigned int*)(((char *)&s1)+5);
		i2 = *(unsigned int*)(((char *)&s2)+5);
		return i1 < i2;
	}
};


struct sockaddrLess6
{
	bool operator()(sockaddr_in6 s1, sockaddr_in6 s2)
	{
		unsigned int i1, i2;
		i1 = *(unsigned int*)(((char *)&s1)+5);
		i2 = *(unsigned int*)(((char *)&s2)+5);
		return i1 < i2;
	}
};


class Packet {

	unsigned char packet[MTU];
	ipq_handle *h;	
	ipq_packet_msg_t *pm;

	std::string err;

	sockaddr_in (*choose_ip4)();

public:


	Packet() : h(NULL), pm(NULL), choose_ip4(NULL) { }; // TODO: different MTU

	virtual ~Packet() { };

	int init();

	const char *why() { return err.c_str(); }

	int recv();

	int balance(sockaddr_in &, sockaddr_in6 &, int af);

	int send(const sockaddr *, int af);

};


int fix_ipv4(unsigned char *p, size_t, const sockaddr *);
int fix_ipv6(unsigned char *p, size_t, const sockaddr *);


}

#endif


